@@ -1142,7 +1142,7 @@ class _AnnotateViewControl(_AbstractViewControl):
                                 s.style(), s.paper())
         self._sci.SendScintilla(qsci.SCI_STYLESETFONT,
                                 s.style(),
-                                unicode(s.font().family()).encode('latin-1'))
+                                unicode(s.font().family()).encode('utf-8'))
         self._sci.SendScintilla(qsci.SCI_STYLESETSIZE,
                                 s.style(), s.font().pointSize())
         for i, (fctx, _origline) in enumerate(self._links):
