@@ -410,7 +410,7 @@ class _ChangesetsModel(QAbstractTableModel):
             return rev in self._selectedrevs and Qt.Checked or Qt.Unchecked
         if role == Qt.DisplayRole:
             coldata = self._COLUMNS[index.column()][1]
-            return hglib.tounicode(coldata(self._repo.changectx(rev)))
+            return hglib.tounicode(coldata(self._repo[rev]))
 
         return None
 
