@@ -6,24 +6,53 @@
 # This software may be used and distributed according to the terms
 # of the GNU General Public License, incorporated herein by reference.
 
+from __future__ import absolute_import
+
 import os
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
 
-from mercurial import error, util, revset as hgrevset
-from mercurial import repoview
+from .qtcore import (
+    QEvent,
+    QTimer,
+    Qt,
+    pyqtSignal,
+    pyqtSlot,
+)
+from .qtgui import (
+    QCheckBox,
+    QComboBox,
+    QCompleter,
+    QFontMetrics,
+    QLabel,
+    QLineEdit,
+    QMainWindow,
+    QMenu,
+    QSizePolicy,
+    QStyle,
+    QToolBar,
+    QToolButton,
+)
 
-from tortoisehg.util import hglib
-from tortoisehg.util.i18n import _
-from tortoisehg.hgqt import revset, qtlib
+from mercurial import (
+    error,
+    repoview,
+    util,
+)
+
+from ..util import hglib
+from ..util.i18n import _
+from . import (
+    qtlib,
+    revset,
+)
 
 _permanent_queries = ('head()', 'merge()',
                       'tagged()', 'bookmark()',
                       'file(".hgsubstate") or file(".hgsub")')
 
 def _firstword(query):
+    lquery = hglib.fromunicode(query)
     try:
-        for token, value, _pos in hgrevset.tokenize(hglib.fromunicode(query)):
+        for token, value, _pos in hglib.tokenizerevspec(lquery):
             if token == 'symbol' or token == 'string':
                 return value  # localstr
     except error.ParseError:
@@ -107,14 +136,14 @@ class RepoFilterBar(QToolBar):
         self._repoagent = repoagent
         self._permanent_queries = list(_permanent_queries)
         repo = repoagent.rawRepo()
-        username = repo.ui.config('ui', ['username', 'user'])
+        username = hglib.configuredusername(repo.ui)
         if username:
             self._permanent_queries.insert(0,
-                hgrevset.formatspec('author(%s)', os.path.expandvars(username)))
+                hglib.formatrevspec('author(%s)', os.path.expandvars(username)))
         self.filterEnabled = True
 
         #Check if the font contains the glyph needed by the branch combo
-        if not QFontMetrics(self.font()).inFont(QString(u'\u2605').at(0)):
+        if not QFontMetrics(self.font()).inFont(u'\u2605'):
             self._allBranchesLabel = u'*** %s ***' % _('Show all')
 
         self.entrydlg = revset.RevisionSetQuery(repoagent, self)
@@ -123,6 +152,7 @@ class RepoFilterBar(QToolBar):
 
         self.revsetcombo = combo = QComboBox()
         combo.setEditable(True)
+        combo.setInsertPolicy(QComboBox.NoInsert)
         # don't calculate size hint from history contents, just use as much
         # space as possible. this way, the branch combo can be enlarged up
         # to its preferred width.
@@ -166,8 +196,7 @@ class RepoFilterBar(QToolBar):
         self.editorBtn.clicked.connect(self.openEditor)
         self.addWidget(self.editorBtn)
 
-        icon = QIcon()
-        icon.addPixmap(QApplication.style().standardPixmap(QStyle.SP_TrashIcon))
+        icon = self.style().standardIcon(QStyle.SP_TrashIcon)
         self.deleteBtn = QToolButton()
         self.deleteBtn.setIcon(icon)
         self.deleteBtn.setToolTip(_('Delete selected query from history'))
@@ -261,11 +290,13 @@ class RepoFilterBar(QToolBar):
     def _prepareQuery(self):
         query = unicode(self.revsetcombo.currentText()).strip()
         if _querytype(self._repo, query) == 'keyword':
-            s = hglib.fromunicode(query)
-            return hglib.tounicode(hgrevset.formatspec('keyword(%s)', s))
+            return hglib.formatrevspec('keyword(%s)', query)
         else:
             return query
 
+    def _isUnsavedQuery(self):
+        return unicode(self.revsetcombo.currentText()).startswith(' ')
+
     @pyqtSlot()
     def _updateQueryType(self):
         query = unicode(self.revsetcombo.currentText()).strip()
@@ -279,6 +310,8 @@ class RepoFilterBar(QToolBar):
             'keyword': (_('Keyword Search'), '#cccccc', '#eeeeee'),
             'revset':  (_('Revision Set'),   '#f6dd82', '#fcf1ca'),
             }[qtype]
+        if self._isUnsavedQuery():
+            name += ' ' + _('(unsaved)')
         label = self._revsettypelabel
         label.setText(name)
         label.setStyleSheet('border: 1px solid %s; background-color: %s; '
@@ -315,6 +348,8 @@ class RepoFilterBar(QToolBar):
 
     def saveQuery(self):
         query = self.revsetcombo.currentText()
+        if self._isUnsavedQuery():
+            return
         if query in self.revsethist:
             self.revsethist.remove(query)
         if query not in self._permanent_queries:
@@ -328,16 +363,17 @@ class RepoFilterBar(QToolBar):
     def loadSettings(self, s):
         repoid = hglib.shortrepoid(self._repo)
         s.beginGroup('revset/' + repoid)
-        self.entrydlg.restoreGeometry(s.value('geom').toByteArray())
-        self.revsethist = list(s.value('queries').toStringList())
-        self.filtercb.setChecked(s.value('filter', True).toBool())
+        self.entrydlg.restoreGeometry(qtlib.readByteArray(s, 'geom'))
+        self.revsethist = list(qtlib.readStringList(s, 'queries'))
+        self.filtercb.setChecked(qtlib.readBool(s, 'filter', True))
         full = self.revsethist + self._permanent_queries
         self.revsetcombo.clear()
         self.revsetcombo.addItems(full)
         self.revsetcombo.setCurrentIndex(-1)
-        self.setVisible(s.value('showrepofilterbar').toBool())
-        self.showHiddenBtn.setChecked(s.value('showhidden').toBool())
-        self.showGraftSourceBtn.setChecked(s.value('showgraftsource', True).toBool())
+        self.setVisible(qtlib.readBool(s, 'showrepofilterbar'))
+        self.showHiddenBtn.setChecked(qtlib.readBool(s, 'showhidden'))
+        self.showGraftSourceBtn.setChecked(
+            qtlib.readBool(s, 'showgraftsource', True))
         self._loadBranchFilterSettings(s)
         s.endGroup()
 
@@ -383,6 +419,8 @@ class RepoFilterBar(QToolBar):
         self._branchCombo.setMaxVisibleItems(30)
         self._branchCombo.currentIndexChanged.connect(self._emitBranchChanged)
         completer = QCompleter(self._branchCombo.model(), self._branchCombo)
+        if hasattr(completer, 'setFilterMode'):  # Qt>=5.2
+            completer.setFilterMode(Qt.MatchContains)
         self._branchCombo.setCompleter(completer)
 
         self.addWidget(self._branchLabel)
@@ -390,14 +428,14 @@ class RepoFilterBar(QToolBar):
         self.addWidget(self._branchCombo)
 
     def _loadBranchFilterSettings(self, s):
-        branch = unicode(s.value('branch').toString())
+        branch = qtlib.readString(s, 'branch')
         if branch == '.':
             branch = hglib.tounicode(self._repo.dirstate.branch())
         self._branchCombo.blockSignals(True)
         self.setBranch(branch)
         self._branchCombo.blockSignals(False)
 
-        self._allparAction.setChecked(s.value('branch_allparents').toBool())
+        self._allparAction.setChecked(qtlib.readBool(s, 'branch_allparents'))
 
     def _saveBranchFilterSettings(self, s):
         branch = self.branch()
@@ -453,7 +491,7 @@ class RepoFilterBar(QToolBar):
     def branch(self):
         """Return the current branch name [unicode]"""
         index = self._branchCombo.currentIndex()
-        branch = self._branchCombo.itemData(index).toString()
+        branch = self._branchCombo.itemData(index)
         return unicode(branch)
 
     def branchAncestorsIncluded(self):
