@@ -36,7 +36,10 @@ ui.promptchoice (with ui.prompt)
 
 import time
 
-from mercurial import util
+from mercurial import (
+    error,
+    util,
+)
 
 from tortoisehg.util import hgversion
 from tortoisehg.util.i18n import agettext as _
@@ -198,13 +201,19 @@ def _extenduiclass(parcls):
         _lastprogresstopic = None
         _lastprogresstime = 0
 
-        def write(self, *args, **opts):
-            if self._buffers:
-                # do not label buffered data because it can be written later
-                super(pipeui, self).write(*args, **opts)
-                return
-            label = opts.get('label', '')
-            super(pipeui, self).write(*_packmsgs(args, label), **opts)
+        if util.safehasattr(parcls, '_writenobuf'):
+            def _writenobuf(self, *args, **opts):
+                label = opts.get('label', '')
+                super(pipeui, self)._writenobuf(*_packmsgs(args, label), **opts)
+        else:
+            # hg<4.6 (b62c4154bb28)
+            def write(self, *args, **opts):
+                if self._buffers:
+                    # do not label buffered data because it can be written later
+                    super(pipeui, self).write(*args, **opts)
+                    return
+                label = opts.get('label', '')
+                super(pipeui, self).write(*_packmsgs(args, label), **opts)
 
         def write_err(self, *args, **opts):
             label = opts.get('label', '')
@@ -223,7 +232,7 @@ def _extenduiclass(parcls):
             try:
                 return resps.index(r.lower())
             except ValueError:
-                raise util.Abort(_('unrecognized response: %s') % r)
+                raise error.Abort(_('unrecognized response: %s') % r)
 
         def getpass(self, prompt=None, default=None):
             prompt = self.label(prompt or _('password: '), 'ui.getpass')
